<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

/**
 * dLocal Go Checkout API client
 */
class WC_Gateway_DlocalGo_Client
{

	/**
	 * Merchant API Key
	 */
	protected $_api_key;

	/**
	 * Merchant API Secret
	 */
	protected $_api_secret;

	/**
	 * API URL
	 */

	protected $_api_url;

	/**
	 * Default country
	 */

	protected $_default_country;

	/**
	 * Constructor.
	 *
	 * @param $_api_key  Merchant's API Key
	 * @param $_api_secret Merchant's API Secret
	 *
	 */
	public function __construct($_api_key, $_api_secret, $_api_url, $_default_country)
	{
		$this->set_api_key($_api_key);
		$this->set_api_secret($_api_secret);
		$this->set_api_url($_api_url);
		$this->set_default_country($_default_country);
	}

	/**
	 * Set default country for the client.
	 *
	 * @param $_default_country Merchant's default country
	 */

	public function set_default_country($_default_country)
	{
		$this->_default_country = $_default_country;
	}

	/**
	 * Set API Key for the client.
	 *
	 * @param $_api_key Merchant's API Key
	 */
	public function set_api_key($_api_key)
	{
		$this->_api_key = $_api_key;
	}

	/**
	 * Set API Secret for the client.
	 *
	 * @param $_api_secret Merchant's API Secret
	 */

	public function set_api_secret($_api_secret)
	{
		$this->_api_secret = $_api_secret;
	}

	/**
	 * Set API URL for the client.
	 *
	 * @param $_api_url Merchant's API URL
	 */

	public function set_api_url($_api_url)
	{
		$this->_api_url = $_api_url;
	}

	/**
	 * Get dLocal Go payments endpoint.
	 *
	 * @return string
	 */
	public function get_payments_endpoint()
	{
		$url = $this->_api_url . '/v1/payments/';
		return $url;
	}

	/**
	 * Get dLocal Go refunds endpoint.
	 *
	 * @return string
	 */

	public function get_refunds_endpoint()
	{
		$url = $this->_api_url . '/v1/refunds/';
		return $url;
	}

	/**
	 * Get merchant's default country.
	 *
	 * @return string
	 */

	public function get_default_country()
	{
		return $this->_default_country;
	}


	/**
	 * Make a remote request to dLocal Go API.
	 *
	 * @param  array $body   Request body
	 * @return array         dLocal Go API response
	 */
	protected function _request($endpoint, $method = 'POST', array $body = [])
	{
		$args = array(
			'headers'     => array('Content-Type' => 'application/json; charset=utf-8', 'Authorization' => 'Bearer ' . $this->_api_key . ':' . $this->_api_secret, 'Referer' => get_site_url()),
			'method'      => $method,
			'user-agent'  => __CLASS__,
			'httpversion' => '1.1',
			'timeout'     => 30,
		);

		if ($method == 'POST') {
			$args['body'] = json_encode($body, JSON_UNESCAPED_UNICODE);
			$logger = wc_get_logger();
			$logger->debug('Request: ' . $args['body'], array('source' => 'dlocalgo'));
			$response = wp_safe_remote_post($endpoint, $args);
		} else if ($method == 'GET') {
			$response = wp_safe_remote_get($endpoint, $args);
		}

		if (is_wp_error($response)) {
			throw new Exception(__('There was a problem connecting to dLocal Go.', 'dlocal-go-payments-for-woocommerce'));
		}

		if (empty($response['body'])) {
			throw new Exception(__('dLocal Go did not return any data.', 'dlocal-go-payments-for-woocommerce'));
		}

		$result = wp_remote_retrieve_body($response);
		$result = json_decode($result, true);

		if ($response['response']['code'] === 400) {
			throw new Exception(__($result['message'], 'dlocal-go-payments-for-woocommerce'));
		}

		return $result;
	}

	/**
	 * Returns true if installed WC version is older than 3.0
	 *
	 * @return boolean
	 */
	protected function _is_old_WC()
	{
		return version_compare(WC_VERSION, '3.0', '<');
	}

	/**
	 * Generate a dLocal Go checkout URL for a given WC order.
	 *
	 * @return string
	 */
	public function get_checkout_url($order_id)
	{
		$order = wc_get_order($order_id);
		$successUrl = preg_replace("/^http:/i", "https:", $order->get_checkout_order_received_url());
		//$successUrl = str_replace('&', '%26', $successUrl);
		$body = array(
			'amount' => $order->get_total(),
			'currency' => $order->get_currency(),
			'country' => $this->getCountryCode($order),
			'order_id' => $order->get_id(),
			'description' => 'Order #' . $order->get_id(),
			'payer' => array(
				'id' => $order->get_customer_id() == 0 ? null : $order->get_customer_id(),
				'first_name' => $order->get_billing_first_name(),
				'last_name' => $order->get_billing_last_name(),
				'email' => $order->get_billing_email(),
				'phone' => $order->get_billing_phone(),
				'user_reference' => $order->get_customer_id() == 0 ? null : $order->get_customer_id(),
			),
			'notification_url' => preg_replace("/^http:/i", "https:", add_query_arg('wc-api', 'WC_Gateway_DlocalGo_Plugin', home_url('/'))),
			'success_url' => $successUrl,
			'back_url' => $successUrl,
			"sub_type" => "WOO_COMMERCE",
			"url_source" => get_site_url(),
		);

		//'back_url' => preg_replace("/^http:/i", "https:", $order->get_cancel_order_url_raw()),


		$logger = wc_get_logger();
		$logger->debug('dLocal Go Checkout Request: ' . print_r($body, true), array('source' => 'dlocal-go-checkout'));

		$response = $this->_request($this->get_payments_endpoint(), 'POST', $body);
		return $response['redirect_url'];
	}

	public function getCountryCode($order)
	{
		$countryCode = $order->get_billing_country();
		if (empty($countryCode)) {
			$countryCode = $order->get_shipping_country();
			if (empty($countryCode)) {
				$countryCode = $this->get_default_country();
				if (empty($countryCode)) {
					throw new Exception(__('Country must be completed.', 'dlocal-go-payments-for-woocommerce'));
				}
			}
		}
		return $countryCode;
	}

	/**
	 * Get transaction status
	 *
	 * @return string
	 */
	public function get_payment_info($invoiceId)
	{
		$response = $this->_request($this->get_payments_endpoint() . $invoiceId, 'GET');
		return [
			'status' => $response['status'],
			'order_id' => $response['order_id'],
		];
	}

	/**
	 * Refund a payment.
	 *
	 * @param  int $order_id
	 * @param  float $amount
	 * @param  string $reason
	 * 
	 * @return associative array
	 */
	public function refund($transaction_id, $currency_code, $amount = null)
	{
		$body = [
			'payment_id' => $transaction_id,
			'notification_url' => preg_replace("/^http:/i", "https:", add_query_arg('wc-api', 'WC_Gateway_DlocalGo_Plugin_Refund', home_url('/'))),
			"sub_type" => "WOO_COMMERCE",
			"url_source" => get_site_url(),
			"currency" => $currency_code,
		];
		if ($amount) {
			$body['amount'] = $amount;
		}
		$logger = wc_get_logger();
		$logger->debug('dLocal Go Refund Request: ' . print_r($body, true), array('source' => 'dlocal-go-refund'));
		$response = $this->_request($this->get_refunds_endpoint(), 'POST', $body);
		return [
			'status' => $response['status'],
			'refund_id' => $response['id'],
		];
	}

	/**
	 * 
	 * Get refund status
	 * @param  int $payment_id
	 * 
	 * @return associative array
	 */
	public function get_refund($payment_id)
	{
		$response = $this->_request($this->get_refunds_endpoint() . $payment_id, 'GET');
		return $response;
	}
}
