<?php

/**
 * dLocal Go Checkout Plugin.
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class WC_Gateway_DlocalGo_Plugin extends WC_Payment_Gateway
{

	/**
	 * dLocal Go API Client.
	 */
	protected $_client;

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		$this->id = "dlocal-go-payments-for-woocommerce";
		$this->icon = "https://dashboard.dlocalgo.com/favicon.svg";
		$this->has_fields = false;
		$this->method_title = __('dLocal Go Checkout', 'dlocal-go-payments-for-woocommerce');
		$this->method_description = __('dLocal Go Checkout redirects customers to dLocal Go to generate local payments', 'dlocal-go-payments-for-woocommerce');

		// Define user set variables
		$this->title        = $this->get_option('title');
		$this->description  = $this->get_option('description');
		$this->api_key       = $this->get_option('api_key');
		$this->api_secret    = $this->get_option('api_secret');
		$this->api_key_sandbox       = $this->get_option('api_key_sandbox');
		$this->api_secret_sandbox    = $this->get_option('api_secret_sandbox');
		$this->production = $this->get_option('production') === 'yes' ? true : false;
		$this->autocomplete_paid_orders = $this->get_option('autocomplete_paid_orders');
		$this->default_country = $this->get_option('default_country');

		$this->supports = array('products', 'refunds',);

		$this->init_form_fields();
		$this->init_settings();

		$this->_load_client();

		// Actions
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
		add_action('woocommerce_api_wc_gateway_dlocalgo_plugin', array($this, 'check_ipn_response'));
		add_action('woocommerce_cancelled_order', array($this, 'cancel_order'));
		add_action('woocommerce_api_wc_gateway_dlocalgo_plugin_refund', array($this, 'check_refund_ipn_response'));

		if ($this->autocomplete_paid_orders) {
			add_action('woocommerce_payment_complete_order_status', array($this, 'wc_auto_complete_paid_order'), 10, 3);
		}
	}


	public function cancel_order($order_id)
	{
		$logger = wc_get_logger();
		$context = array('source' => 'prueba');
		$logger->debug('cancel_order', $context);
		$order = wc_get_order($order_id);
		foreach ($order->get_items() as $item) {
			$product_id = $item->get_product_id();
			$quantity = $item->get_quantity();
			$variation_id = $item->get_variation_id();
			$variation = array();
			$cart_item_data = array();
			if ($variation_id) {
				$variation = wc_get_product_variation_attributes($variation_id);
			}
			WC()->cart->add_to_cart($product_id, $quantity, $variation_id, $variation, $cart_item_data);
		}
		wp_redirect($order->get_cancel_order_url_raw());
	}

	public function can_refund_order($order)
	{
		if ($order->get_payment_method() === $this->id) {
			return $order && $order->get_transaction_id();
		}
		return false;
	}

	public function process_refund($order_id, $amount = null, $reason = '')
	{
		try {
			$order = wc_get_order($order_id);
			if (!$this->can_refund_order($order)) {
				return new WP_Error('error', __('Refund Failed: No transaction ID', 'dlocal-go-payments-for-woocommerce'));
			}
			$transaction_id = $order->get_transaction_id();
			$currencyCode = $order->get_currency();
			$refund = $this->_client->refund($transaction_id, $currencyCode, $amount);
			$order->add_order_note(
				sprintf(
					__('Refund processed via dLocal Go. Refund ID: %s - Refund amount: %s - Refund status: %s - Refund reason: %s', 'dlocal-go-payments-for-woocommerce'),
					$refund['refund_id'],
					$amount,
					$refund['status'],
					$reason
				)
			);
			if ($refund['status'] === 'PENDING' || $refund['status'] === 'CREATED') {
				return new WP_Error('error', __('Refund was created succesfully, it is pending', 'dlocal-go-payments-for-woocommerce'));
			}
			return $refund['status'] === 'COMPLETED' || $refund['status'] === 'SUCCESS';
		} catch (\Exception $e) {
			$logger = wc_get_logger();
			$context = array('source' => 'refund error');
			$logger->debug($e->getMessage(), $context);
			$order->add_order_note(
				sprintf(
					__('Refund Failed: %s', 'dlocal-go-payments-for-woocommerce'),
					"Unknown error"
				)
			);
			return new WP_Error('dlocal-go-refund-error', "Unable to process refund, an unknown error occurred.");
		}
	}

	public function check_refund_ipn_response()
	{
		$body = file_get_contents('php://input');
		$params = json_decode($body, true);
		$refund_id = $params['id'];
		if ($refund_id) {
			$refund = $this->_client->get_refund($refund_id);
			$order = wc_get_order($refund['order_id']);
			$order->add_order_note(
				sprintf(
					__('Refund ID: %s - Refund status: %s - Refund amount: %s', 'dlocal-go-payments-for-woocommerce'),
					$refund['refund_id'],
					$refund['status'],
					$refund['amount']
				)
			);
			if ($order->get_status() === 'completed' && ($refund['status'] === 'COMPLETED' || $refund['status'] === 'SUCCESS')) {
				$refund = wc_create_refund(array(
					'amount'     => $refund['amount'],
					'reason'     => 'Refund via dLocal Go',
					'order_id'   => $order->get_id(),
					'line_items' => array(),
					'refund_payment' => false,
				));
			}
		}
	}



	protected function _load_client()
	{
		require_once(dirname(__FILE__) . '/class-wc-gateway-dlocalgo-client.php');
		$apiKey = $this->production ? $this->api_key : $this->api_key_sandbox;
		$apiSecret = $this->production ? $this->api_secret : $this->api_secret_sandbox;
		$apiUrl = $this->production ? 'https://api.dlocalgo.com' : 'https://api-sbx.dlocalgo.com';
		$defaultCountry = $this->default_country;
		$this->_client = new WC_Gateway_DlocalGo_Client($apiKey, $apiSecret, $apiUrl, $defaultCountry);
	}

	/**
	 * Initialize Gateway Settings Form Fields
	 */
	public function init_form_fields()
	{
		$countries_obj   = new WC_Countries();
		$countries   = $countries_obj->__get('countries');

		$this->form_fields = apply_filters('wc_dlocalgo_form_fields', array(

			'enabled' => array(
				'title'   => __('Enable/Disable', 'dlocal-go-payments-for-woocommerce'),
				'type'    => 'checkbox',
				'label'   => __('Enable dLocal Go Checkout', 'dlocal-go-payments-for-woocommerce'),
				'default' => 'yes'
			),

			'title' => array(
				'title'       => __('Title', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'text',
				'description' => __('This controls the title for the payment method the customer sees during checkout.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => __('dLocal Go', 'dlocal-go-payments-for-woocommerce'),
				'desc_tip'    => true,
			),

			'description' => array(
				'title'       => __('Description', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'text',
				'description' => __('Payment method description that the customer will see on your checkout.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => __('Pay using dLocal Go Checkout.', 'dlocal-go-payments-for-woocommerce'),
				'desc_tip'    => true,
			),

			'api_key_sandbox' => array(
				'title'       => __('Api Key Sandbox', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'password',
				'description' => __('Your sandbox dLocal Go API key.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => '',
				'desc_tip'    => true,
			),

			'api_secret_sandbox' => array(
				'title'       => __('Api Secret Sandbox', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'password',
				'description' => __('Your sandbox dLocal Go API secret.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => '',
				'desc_tip'    => true,
			),

			'production' => array(
				'title'   => __('Production', 'dlocal-go-payments-for-woocommerce'),
				'type'    => 'checkbox',
				'label'   => __('Yes', 'dlocal-go-payments-for-woocommerce'),
				'default' => 'yes'
			),

			'api_key' => array(
				'title'       => __('Production Api Key', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'password',
				'description' => __('Your production dLocal Go API key.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => '',
				'desc_tip'    => true,
			),

			'api_secret' => array(
				'title'       => __('Production Api Secret', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'password',
				'description' => __('Your production dLocal Go API secret.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => '',
				'desc_tip'    => true,
			),

			'autocomplete_paid_orders' => array(
				'title'   => __('Auto Complete Paid Orders', 'dlocal-go-payments-for-woocommerce'),
				'type'    => 'checkbox',
				'label'   => __('Yes', 'dlocal-go-payments-for-woocommerce'),
				'default' => 'yes'
			),
			'default_country' => array(
				'title'       => __('Default Fallback Country', 'dlocal-go-payments-for-woocommerce'),
				'type'        => 'select',
				'description' => __('Default country to send to dLocal Go when billing address and shipping address country is empty.', 'dlocal-go-payments-for-woocommerce'),
				'default'     => '',
				'desc_tip'    => true,
				'options' => array(
					'' => __('None', 'dlocal-go-payments-for-woocommerce')) + $countries,
			
			),
		));
	}

	/**
	 * Process the payment and return the result
	 *
	 * @param int $order_id
	 * @return array
	 */
	public function process_payment($order_id)
	{
		$redirect_url = '';
		try {
			$redirect_url = $this->_client->get_checkout_url($order_id);
		} catch (Exception $e) {
			wc_add_notice($e->getMessage(), 'error');
			return;
		}

		if (is_null($redirect_url)) {
			wc_add_notice(__('No redirect URL', 'dlocal-go-payments-for-woocommerce'), 'error');
			return;
		}

		WC()->cart->empty_cart();

		return array(
			'result'   => 'success',
			'redirect' => $redirect_url
		);
	}

	/**
	 * Validate IPN response
	 */
	public function check_ipn_response()
	{
		$body = file_get_contents('php://input');
		$params = json_decode($body, true);
		$payment_id = $params['payment_id'];
		if ($payment_id) {
			$paymentInfo = $this->_client->get_payment_info($payment_id);
			$order = new WC_Order($paymentInfo['order_id']);
			if ($order && $order->get_status() !== 'completed' && $paymentInfo['status'] == 'PAID') {
				$order->payment_complete($payment_id);
				$order->add_order_note(__('dLocal Go payment completed', 'dlocal-go-payments-for-woocommerce'));
			} else if ($order &&  $order->get_status() !== 'failed' && $paymentInfo['status'] == 'REJECTED') {
				$order->update_status('failed', __('dLocal Go payment rejected', 'dlocal-go-payments-for-woocommerce'));
			} else if ($order &&  $order->get_status() !== 'cancelled' && $paymentInfo['status'] == 'CANCELLED') {
				$order->update_status('cancelled', __('dLocal Go payment cancelled', 'dlocal-go-payments-for-woocommerce'));
			} else if ($order &&  $order->get_status() !== 'failed' && $paymentInfo['status'] == 'EXPIRED') {
				$order->update_status('failed', __('dLocal Go payment refunded', 'dlocal-go-payments-for-woocommerce'));
			}
		}
	}

	public function wc_auto_complete_paid_order($status, $order_id, $order)
	{
		$logger = wc_get_logger();
		$logger->debug('wc_auto_complete_paid_order', array('source' => 'dlocal-go-payments-for-woocommerce'));
		$logger->debug($status, array('source' => 'dlocal-go-payments-for-woocommerce'));
		$logger->debug($order->get_status(), array('source' => 'dlocal-go-payments-for-woocommerce'));
		if ($order->get_status() == 'cancelled') {
			return;
		}
		if ($status == 'processing' && $this->get_option('autocomplete_paid_orders') == 'yes') {
			if ($order->get_payment_method() == $this->id) {
				$order->update_status('completed', __('Order completed by dLocal Go', 'dlocal-go-payments-for-woocommerce'));
				$status = 'completed';
			}
		}
		return $status;
	}
}
